/*
 * Copyright (C) 2000-2005 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 * $Id: btreceiver.c,v 1.16 2007/02/15 16:30:49 freckle Exp $
 */
/*!
 * $Log: btreceiver.c,v $
 * Revision 1.16  2007/02/15 16:30:49  freckle
 * improved bt_hci_read_clock -- adapted API
 *
 * Revision 1.15  2007/02/12 18:22:12  freckle
 * update bt_hci_read_clock call to current API
 *
 * Revision 1.14  2007/02/12 18:08:28  freckle
 * use new CL ACL AP
 *
 * Revision 1.12.6.1  2007/01/09 16:57:56  freckle
 * removed COD event filter
 *
 * Revision 1.13  2006/10/27 14:53:52  yuecelm
 * Changed signedness of strings in order to compile with avr-gcc >4.0
 *
 * Revision 1.12  2006/04/06 08:53:07  kevmarti
 * Adapted to new interface of HCI con table callback & HCI acl callback
 *
 * Revision 1.11  2006/03/23 07:22:25  dyerm
 * Merged changes from multihop_merge branch. See individual changes on multihop_merge branch. See Changelog for summary of changes.
 *
 * Revision 1.9.2.1  2006/02/02 08:09:37  dyerm
 * meged changes from MAIN branch from [Root_multihop_merge] to [multihop_merged_1] into multihop_merge branch
 *
 * Revision 1.10  2005/12/06 18:53:28  beutel
 * fixed links on api doc's for new webpage, cleaned examples docu
 *
 * Revision 1.9  2005/10/05 15:43:40  beutel
 * minor adaption for doxygen and 1.4 release
 *
 */

/**
 * \example btstreamer/receiver/btreceiver.c
 *
 * \date 2004/04/18 21:37:09 
 * 
 * \author Mathias Payer <payerm@student.ethz.ch>
 *
 * Example application to show the use of the bt stack
 */

#ifdef __BTN_UNIX__
#include <dev/unix_devs.h>
#include <unistd.h>
#endif

#include <stdio.h>
#include <string.h>
#include <io.h>
#include <stdlib.h>
#include <dev/debug.h>
#include <sys/timer.h>
#include <sys/event.h>
#include <dev/uartavr.h>
#include <dev/usartavr.h>
#include <bt/bt_hci_cmds.h>
#include <hardware/btn-hardware.h>
#include <led/btn-led.h>


/** Number of ACL data-buffers, at least 1 for null-(drop)-function
* a buffer of 8 packets seems ok, then there can be 8 open conns at a
* time, that are waiting for more data.
*/
#define HCI_MAX_ACL_DATA_PKT	0x08
/** Number of SCO data-buffers, at least 1 for null-(drop)-function */
#define HCI_MAX_SCO_DATA_PKT	0x01

#define HCI_PACKET_TYPE BT_HCI_PACKET_TYPE_DH3

//Global variables on the heap
struct btstack *stack;
struct bt_hci_inquiry_result inqres[5];
struct bt_hci_pkt_acl *aclpkt;

bt_hci_con_handle_t THE_handle;
HANDLE table_changed_event;
HANDLE ten_event;

u_short packet_counter;

void change_con_pkt_type(bt_hci_con_handle_t con_handle) {
    long res;
    printf("change con pkt type to 0x%.4x\n",HCI_PACKET_TYPE);
    res = bt_hci_change_con_pkt_type(stack, BT_HCI_SYNC, con_handle, HCI_PACKET_TYPE);
    if (res < 0)
        printf("pkt change: error\n");
    else {
        printf("pkt changed to 0x%.4lx\n",res);
    }
}


void con_table_cb(u_char type, u_char reason, bt_hci_con_handle_t con_handle, void* cb_arg)
{ 
    printf("\nconnection table changed!\n");
    switch (type) {
        case BT_HCI_MY_ROLE_MASTER:
            printf("CB: My role switched to master (%d)\n",con_handle);
            break;
        case BT_HCI_MY_ROLE_SLAVE:
            printf("CB: My role switched to slave (%d)\n",con_handle);
            break;
        case BT_HCI_CONNECTION:
            printf("CB: Connection opened (%d)\n",con_handle);
            THE_handle = con_handle;
            NutEventPostAsync(&table_changed_event);
            break;
        case BT_HCI_DISCONNECTION:
            printf("CB: Disconnected (%d)\n",con_handle);
            break;
        default:
            printf("Illegal type entered! (%d)\n",type);
    }
}


struct bt_hci_pkt_acl *stream_receive(void *arg, struct bt_hci_pkt_acl *pkt, bt_hci_con_handle_t app_con_handle,
                                u_char pb_flag, u_char bc_flag, u_short len, u_long t_arrive)
{
	long retval;
	u_short acllen, i;
	// printf("Acl pkt. from handle %d with pb=%d, bc=%c of len %d dropped\n\r", app_con_handle, pb_flag, bc_flag, len);
	
	acllen = pkt->payload[0] | pkt->payload[1]<<8;
	if (acllen != 183) { printf("Wrong ACL packet length: %d\n", acllen); }
	
	for (i=10; i<183; i++) {
		if (pkt->payload[i] != i-2) { printf("Wrong data at position %d: %d\n", i, pkt->payload[i]); }
	}

	/* DEBUG("Data: %d\n", acllen);
	for (i=0; i<acllen-2; i++) {
		DEBUG("%c", pkt->payload[i+2]);
	}
	DEBUG("\n"); */
	
	if (++packet_counter==10) {
		printf(".");
		packet_counter = 0;
        // NutEventPostAsync(&ten_event);
	}

	// Free this pkt!
	retval = bt_hci_acl_free_buffer((struct btstack*)arg, app_con_handle, 1);
    // printf("pkt freed: %ld \n\r", retval);
	return pkt;
}

int main(void)
{
	long retval;
	u_long baud = 57600;
	char name[64];
	u_long myTime;
    u_long masterTime;
    
    // hardware init
    btn_hardware_init();
    btn_led_init(1);
	
    // init app uart
    NutRegisterDevice(&APP_UART, 0, 0);
    freopen(APP_UART.dev_name, "r+", stdout);
    _ioctl(_fileno(stdout), UART_SETSPEED, &baud);

    // bluetooth module on (takes a while)
    btn_hardware_bt_on();
	
	// Start the stack and let the initializaiton begin
	stack = bt_hci_init(&BT_UART);
	
	// Set the speed to 57600
	// retval = bt_hci_set_baudrate(stack, BT_HCI_B57600);
	// if (retval != 0) printf("Setting the baudrate to 115200: %ld\n", retval);
	// Set the speed to 115200
	// retval = bt_hci_set_baudrate(stack, BT_HCI_B115200);
	// if (retval != 0) printf("Setting the baudrate to 115200: %ld\n", retval);
	// Set the speed to 230400
	// retval = bt_hci_set_baudrate(stack, BT_HCI_B230400);
	// if (retval != 0) printf("Setting the baudrate to BT_HCI_B230400: %ld\n", retval);
	
	// add connection handling to allow role change
	bt_hci_register_con_table_cb(stack, &con_table_cb, NULL);
	
	// prepare data reception
    aclpkt =(struct bt_hci_pkt_acl *) malloc( 200 );
	bt_hci_register_cl_acl_cb(stack, &stream_receive, aclpkt, stack);
	
	// From now on the stack should be working...
	
	strcpy(name, "BT-Receiver");
	retval = bt_hci_write_local_name(stack, BT_HCI_SYNC, (u_char*) name);
	if (retval!=0) printf("Setting the btname: %ld\n", retval);
	
	// Dirty hack:
	// We have only one packet, and the max.size is BT_HCI_MAX_DATA_PACKET
	// but we know that it will never be fragmented!
	retval = bt_hci_host_buffer_size(stack, BT_HCI_SYNC, 600, 0, 8, 0);
	printf("Set host buffer size: %ld\n", retval);
	
	// Loop forever
	printf("Entering main loop, ready to receive data!\n");
	
	// Loop forever
	table_changed_event = 0;
	packet_counter = 0;
	while (1) {
		printf("LOOP (MAIN)\n");
		// change connection pkt type and link policy for incoming connections
		// does not handle multiple incoming connections in short time
		NutEventWait(&table_changed_event,NUT_WAIT_INFINITE);
		//change_con_pkt_type(THE_handle);
		retval = bt_hci_write_link_policy_settings(stack, BT_HCI_SYNC, THE_handle, BT_HCI_LINK_POLICY_ROLE_SWITCH);
        
        while ( 1 ) {
            NutEventWait( &ten_event,NUT_WAIT_INFINITE );
            myTime = bt_hci_read_clock( stack, BT_HCI_SYNC, 0, 0, NULL, NULL);
            retval = bt_hci_read_clock_offset( stack, BT_HCI_SYNC, THE_handle);
            if (retval < 16484 )
                masterTime = myTime - retval;
            else
                masterTime = myTime - retval + 32768;
            printf("%.6lu %.6lu %.6lu \r", masterTime, myTime, retval);
        }
	}
	return 0;
}
