/*
 * Copyright (C) 2000-2007 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 * $Id: suart-echo.c,v 1.1 2007/02/02 10:43:11 yuecelm Exp $
 * 
 */

/*!
 * $Log: suart-echo.c,v $
 * Revision 1.1  2007/02/02 10:43:11  yuecelm
 * add app suart-echo
 *
 *
 */ 

/**
 * \example suart-echo/suart-echo.c
 *
 * \date 03.01.2007
 * 
 * \author Martin Hinz <btnode@hinz.ch>
 * \author Mustafa Yuecel <mustafa.yuecel@alumni.ethz.ch>
 * 
 * This application echos the software uart and signals a
 * received byte with an led pattern
 */
 
#include <string.h>
#include <stdio.h>
#include <io.h>
#include <hardware/btn-hardware.h>
#include <suart/btn-suart.h>
#include <led/btn-led.h>
#include <sys/thread.h>
#include <sys/timer.h>
FILE *suart;


/*
 * listen on suart and write to suart
 */
void listen_to_suart(void)
{
    int got;
    char inbuf[20]; 
    for(;;){
        got = _read(_fileno(suart), inbuf, 20);
        btn_led_clear_pattern_queue();
        btn_led_add_pattern(BTN_LED_PATTERN_UP, 0,1, 1);
        btn_led_add_pattern(BTN_LED_PATTERN_KNIGHT, 0, 4, BTN_LED_INFINITE);
        if (got != -1) {
            _write(_fileno(suart), inbuf, got);
        } else {
            btn_led_clear_pattern_queue();
            btn_led_set(1); // turn on red led
            NutSleep(1000);
            btn_led_clear(1); // turn off red led
        }
    }
}

int main(void)
{   
    u_long baud = 2400;

    btn_hardware_init();

    //initialize software UART
    NutRegisterDevice(&S_UART, 0, 0);
    suart = fopen(S_UART.dev_name, "r+");
    _ioctl(_fileno(suart), UART_SETSPEED, &baud);

    fputs("SUART-echo application!\n",suart); 
    btn_led_init(1);
    btn_led_add_pattern(BTN_LED_PATTERN_KNIGHT, 0, 4, BTN_LED_INFINITE);
    
    listen_to_suart();
    return 1;
}
