/* Copyright (c) 2002, Marek Michalkiewicz
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
   * Neither the name of the copyright holders nor the names of
     contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. */

#if defined(__AVR_ATmega128__)

#include "macros.inc"
#include <arch/btn-hardware.h>

#define dest_hi r25
#define dest_lo r24
#define src_hi r23
#define src_lo r22
#define len_hi r21
#define len_lo r20

/** \ingroup avr_string
    \fn void *memcpy(void *dest, const void *src, size_t len)
    \brief Copy a memory area.

    The memcpy() function copies len bytes from memory area src to memory area
    dest.  The memory areas may not overlap.  Use memmove() if the memory
    areas do overlap.

    \returns The memcpy() function returns a pointer to dest.  */



;cpy from bank 0 to bank 1
	.text
	.global	_U(bank_cpy_01)
	.type	_U(bank_cpy_01), @function
_U(bank_cpy_01):
	LOAD_Z(src_lo, src_hi)
	LOAD_X(dest_lo, dest_hi)

	clt			       ; clear transfer bit
	brbc    7, .bank_cpy_01_start  ; start
	set			       ; set transfer bit if interrupt enabled
	rjmp	.bank_cpy_01_start
.bank_cpy_01_loop:
	cli
	ld	__tmp_reg__, Z+
	sbi     RAM_CTR_A_PORT-0x20, RAM_CTR_A_PIN
	st	X+, __tmp_reg__
	cbi     RAM_CTR_A_PORT-0x20, RAM_CTR_A_PIN
	//brtc    .bank_cpy_01_start     ; skip reenabling interrupt
	sei
.bank_cpy_01_start:
	subi	len_lo, lo8(1)
	sbci	len_hi, hi8(1)

	brcc	.bank_cpy_01_loop

	ret
.bank_cpy_01_end:
	.size	_U(bank_cpy_01), .bank_cpy_01_end - _U(bank_cpy_01)


;cpy from bank 0 to bank 2
	.text
	.global	_U(bank_cpy_02)
	.type	_U(bank_cpy_02), @function
_U(bank_cpy_02):
	LOAD_Z(src_lo, src_hi)
	LOAD_X(dest_lo, dest_hi)

	clt			       ; clear transfer bit
	brbc    7, .bank_cpy_02_start  ; start
	set			       ; set transfer bit if interrupt enabled
	rjmp	.bank_cpy_02_start
.bank_cpy_02_loop:
	cli
	ld	__tmp_reg__, Z+
	sbi     RAM_CTR_B_PORT-0x20, RAM_CTR_B_PIN
	st	X+, __tmp_reg__
	cbi     RAM_CTR_B_PORT-0x20, RAM_CTR_B_PIN
	//brtc    .bank_cpy_02_start     ; skip reenabling interrupt
	sei
.bank_cpy_02_start:
	subi	len_lo, lo8(1)
	sbci	len_hi, hi8(1)

	brcc	.bank_cpy_02_loop

	ret
.bank_cpy_02_end:
	.size	_U(bank_cpy_02), .bank_cpy_02_end - _U(bank_cpy_02)



;cpy from bank 0 to bank 3
	.text
	.global	_U(bank_cpy_03)
	.type	_U(bank_cpy_03), @function
_U(bank_cpy_03):
	LOAD_Z(src_lo, src_hi)
	LOAD_X(dest_lo, dest_hi)

	clt			       ; clear transfer bit
	brbc    7, .bank_cpy_03_start  ; start
	set			       ; set transfer bit if interrupt enabled
	rjmp	.bank_cpy_03_start
.bank_cpy_03_loop:
	cli
	ld	__tmp_reg__, Z+
	sbi     RAM_CTR_A_PORT-0x20, RAM_CTR_A_PIN
	sbi     RAM_CTR_B_PORT-0x20, RAM_CTR_B_PIN
	st	X+, __tmp_reg__
	cbi     RAM_CTR_A_PORT-0x20, RAM_CTR_A_PIN
	cbi     RAM_CTR_B_PORT-0x20, RAM_CTR_B_PIN
	//brtc    .bank_cpy_03_start     ; skip reenabling interrupt
	sei
.bank_cpy_03_start:
	subi	len_lo, lo8(1)
	sbci	len_hi, hi8(1)

	brcc	.bank_cpy_03_loop

	ret
.bank_cpy_03_end:
	.size	_U(bank_cpy_03), .bank_cpy_03_end - _U(bank_cpy_03)	

	
;cpy from bank 1 to bank 0	
	.text
	.global	_U(bank_cpy_10)
	.type	_U(bank_cpy_10), @function
_U(bank_cpy_10):
	LOAD_Z(dest_lo, dest_hi)
	LOAD_X(src_lo, src_hi)

	clt			       ; clear transfer bit
	brbc    7, .bank_cpy_10_start  ; start
	set			       ; set transfer bit if interrupt enabled
	rjmp	.bank_cpy_10_start
.bank_cpy_10_loop:
	cli
	sbi     RAM_CTR_A_PORT-0x20, RAM_CTR_A_PIN
	ld	__tmp_reg__, Z+
	cbi     RAM_CTR_A_PORT-0x20, RAM_CTR_A_PIN
	st	X+, __tmp_reg__
	//brtc    .bank_cpy_10_start     ; skip reenabling interrupt
	sei
.bank_cpy_10_start:
	subi	len_lo, lo8(1)
	sbci	len_hi, hi8(1)

	brcc	.bank_cpy_10_loop

	ret
.bank_cpy_10_end:
	.size	_U(bank_cpy_10), .bank_cpy_10_end - _U(bank_cpy_10)


;cpy from bank 2 to bank 0	
	.text
	.global	_U(bank_cpy_20)
	.type	_U(bank_cpy_20), @function
_U(bank_cpy_20):
	LOAD_Z(dest_lo, dest_hi)
	LOAD_X(src_lo, src_hi)

	clt			       ; clear transfer bit
	brbc    7, .bank_cpy_20_start  ; start
	set			       ; set transfer bit if interrupt enabled
	rjmp	.bank_cpy_20_start
.bank_cpy_20_loop:
	cli
	sbi     RAM_CTR_B_PORT-0x20, RAM_CTR_B_PIN
	ld	__tmp_reg__, Z+
	cbi     RAM_CTR_B_PORT-0x20, RAM_CTR_B_PIN
	st	X+, __tmp_reg__
	//brtc    .bank_cpy_20_start     ; skip reenabling interrupt
	sei
.bank_cpy_20_start:
	subi	len_lo, lo8(1)
	sbci	len_hi, hi8(1)

	brcc	.bank_cpy_20_loop

	ret
.bank_cpy_20_end:
	.size	_U(bank_cpy_20), .bank_cpy_20_end - _U(bank_cpy_20)


;cpy from bank 3 to bank 0	
	.text
	.global	_U(bank_cpy_30)
	.type	_U(bank_cpy_30), @function
_U(bank_cpy_30):
	LOAD_Z(dest_lo, dest_hi)
	LOAD_X(src_lo, src_hi)

	clt			       ; clear transfer bit
	brbc    7, .bank_cpy_30_start  ; start
	set			       ; set transfer bit if interrupt enabled
	rjmp	.bank_cpy_30_start
.bank_cpy_30_loop:
	cli
	sbi     RAM_CTR_A_PORT-0x20, RAM_CTR_A_PIN
	sbi     RAM_CTR_B_PORT-0x20, RAM_CTR_B_PIN
	ld	__tmp_reg__, Z+
	cbi     RAM_CTR_A_PORT-0x20, RAM_CTR_A_PIN
	cbi     RAM_CTR_B_PORT-0x20, RAM_CTR_B_PIN
	st	X+, __tmp_reg__
	//brtc    .bank_cpy_30_start     ; skip reenabling interrupt
	sei
.bank_cpy_30_start:
	subi	len_lo, lo8(1)
	sbci	len_hi, hi8(1)

	brcc	.bank_cpy_30_loop

	ret
.bank_cpy_30_end:
	.size	_U(bank_cpy_30), .bank_cpy_30_end - _U(bank_cpy_30)

#endif
