/*
 * Copyright (C) 2000-2004 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */
 
 /**
 * @file cc/cc1000_defs.h
 *
 * @brief Chipcon CC1000 register definitions
 * 
 *
 * 2004/06/19 Matthias Ringwald <mringwal@inf.ethz.ch>
 *
 */

#ifndef _CC_CC1000_DEFS_H_
#define _CC_CC1000_DEFS_H_

// The registers defined in the CC1000 data sheet

#define CC1000_MAIN            0x00
#define CC1000_FREQ_2A         0x01
#define CC1000_FREQ_1A         0x02
#define CC1000_FREQ_0A         0x03
#define CC1000_FREQ_2B         0x04
#define CC1000_FREQ_1B         0x05
#define CC1000_FREQ_0B         0x06
#define CC1000_FSEP1           0x07
#define CC1000_FSEP0           0x08
#define CC1000_CURRENT         0x09
#define CC1000_FRONT_END       0x0A
#define CC1000_PA_POW          0x0B
#define CC1000_PLL             0x0C
#define CC1000_LOCK            0x0D
#define CC1000_CAL             0x0E
#define CC1000_MODEM2          0x0F
#define CC1000_MODEM1          0x10
#define CC1000_MODEM0          0x11
#define CC1000_MATCH           0x12
#define CC1000_FSCTRL          0x13
#define CC1000_FSHAPE7         0x14
#define CC1000_FSHAPE6         0x15
#define CC1000_FSHAPE5         0x16
#define CC1000_FSHAPE4         0x17
#define CC1000_FSHAPE3         0x18
#define CC1000_FSHAPE2         0x19
#define CC1000_FSHAPE1         0x1A
#define CC1000_FSDELAY         0x1B
#define CC1000_PRESCALER       0x1C
#define CC1000_TEST6           0x40
#define CC1000_TEST5           0x41
#define CC1000_TEST4           0x42
#define CC1000_TEST3           0x43
#define CC1000_TEST2           0x44
#define CC1000_TEST1           0x45
#define CC1000_TEST0           0x46

/* Define each pin for the CC1000 */

// The bit definitions in the MAIN register
#define CC1000_RXTX         7
#define CC1000_F_REG		6
#define CC1000_RX_PD		5
#define CC1000_TX_PD		4
#define CC1000_FS_PD		3
#define CC1000_CORE_PD		2
#define CC1000_BIAS_PD		1
#define CC1000_RESET_N		0


// The bit definitions in the CURRENT register
//CURRENT[7-4],  control of current in VCO core for TX and RX
#define CC1000_VCO_CURRENT	4
//control of current in VCO buffer for LO drive
#define CC1000_LO_DRIVE		2
//control of current in VCO buffer for PA
#define CC1000_PA_DRIVE		0


// The bit definitions in the FRONT_END register
#define CC1000_BUF_CURRENT	5
#define CC1000_LNA_CURRENT	3
#define CC1000_IF_RSSI		1
#define CC1000_XOSC_BYPASS	0


// The bit definitions in the PA_POW register
#define CC1000_PA_HIGHPOWER	4
#define CC1000_PA_LOWPOWER	0


// The bit definitions in the PLL register
#define CC1000_EXT_FILTER	7
#define CC1000_REFDIV		3
#define CC1000_ALARM_DISABLE	2
#define CC1000_ALARM_H		1
#define CC1000_ALARM_L		0

// The Bit Definitions in the LOCK register
#define CC1000_LOCK_SELECT	4
#define CC1000_PLL_LOCK_ACCURACY	3
#define CC1000_PLL_LOCK_LENGTH	2
#define CC1000_LOCK_INSTANT	1
#define CC1000_LOCK_CONTINUOUS	0


// Bit Definitions in the CAL register 
#define CC1000_CAL_START	7
#define CC1000_CAL_DUAL		6
#define CC1000_CAL_WAIT		5
#define CC1000_CAL_CURRENT	4
#define CC1000_CAL_COMPLETE	3
#define CC1000_CAL_ITERATE	0  //3 bits

// Bit definitions in the MODEM2 register
#define CC1000_PEAKDETECT		7 
#define CC1000_PEAK_LEVEL_OFFSET	0 //7 bits

// Bit Definitions in MODEM1 Register 
#define CC1000_MLIMIT		5
#define CC1000_LOCK_AVG_IN	4
#define CC1000_LOCK_AVG_MODE	3
#define CC1000_SETTLING		1
#define CC1000_MODEM_RESET_N	0

// Bit Definitions in  MODEM0 Register 
#define CC1000_BAUDRATE		4
#define CC1000_DATA_FORMAT	2
#define CC1000_XOSC_FREQ	0

// Bit Definitions in MATCH Register 
#define CC1000_RX_MATCH		4
#define CC1000_TX_MATCH		0

// Bit Definitions in FSCTLR Register 
#define CC1000_DITHER1		3
#define CC1000_DITHER0		2
#define CC1000_SHAPE		1
#define CC1000_FS_RESET_N	0

// Bit Definitions in PRESCALER Register 
#define CC1000_PRE_SWING		6
#define CC1000_PRE_CURRENT	4
#define CC1000_IF_INPUT		3
#define CC1000_IF_FRONT		2

// Bit Definitions in TEST6 Register 
#define CC1000_LOOPFILTER_TP1	7
#define CC1000_LOOPFILTER_TP2	6
#define CC1000_CHP_OVERRIDE	5
#define CC1000_CHP_CO		0

// Bit Definitions in TEST5 Register 
#define CC1000_CHP_DISALBE	5
#define CC1000_VCO_OVERRIDE	4
#define CC1000_VCO_AO		0

// Bit Definitions in TEST3 Register 
#define CC1000_BREAK_LOOP	4
#define CC1000_CAL_DAC_OPEN 0

// Bit Definitons in TEST2 Register 
#define CC1000_CHP_CURRENT  0

// Bit Definitons in TEST1 Register 
#define CC1000_CAL_DAC      0

// Bit Definitons in TEST0 
#define CC1000_VCO_ARRAY    0

// The Number of the total frequencies available to use
#define FREQS_NUM 65 // 30 915 Mhz band, 35 in 868 Mhz band

// The Number of the values defined for each register
#define PARAM_NUM 38
#define SETTING_NUM 6

//the index of the registers in the array
#define MAIN_INDEX	0x00 
#define FREQ2A_INDEX	0x01
#define FREQ1A_INDEX	0x02
#define FREQ0A_INDEX	0x03
#define FREQ2B_INDEX	0x04
#define FREQ1B_INDEX	0x05
#define FREQ0B_INDEX	0x06
#define FSEP1_INDEX	0x07
#define FSEP0_INDEX	0x08
#define RXCURRENT_INDEX	0x09
#define FRONTEND_INDEX	0x0a
#define PAPOW_INDEX	0x0b
#define RXPLL_INDEX	0x0c
#define LOCK_INDEX	0x0d
#define CAL_INDEX	0x0e
#define MODEM2_INDEX	0x0f
#define MODEM1_INDEX	0x10
#define MODEM0_INDEX	0x11
#define MATCH_INDEX	0x12
#define FSCTRL_INDEX	0x13
#define FSHAPE7_INDEX	0x14
#define FSHAPE6_INDEX	0x15
#define FSHAPE5_INDEX	0x16
#define FSHAPE4_INDEX	0x17
#define FSHAPE3_INDEX	0x18
#define FSHAPE2_INDEX	0x19
#define FSHAPE1_INDEX	0x1a
#define FSDELAY_INDEX	0x1b
#define PRESCALER_INDEX	0x1c
#define TEST6_INDEX	0x1d
#define TEST5_INDEX	0x1e
#define TEST4_INDEX	0x1f	
#define TEST3_INDEX	0x20
#define TEST2_INDEX	0x21
#define TEST1_INDEX	0x22
#define TEST0_INDEX	0x23
#define TXCURRENT_INDEX	0x24
#define TXPLL_INDEX	0x25

// Define the transmission speed for the radios 
#define BAUD_0_6_K 0x07     //0.6 KBaud
#define BAUD_1_2_K 0x17     //1.2 KBaud
#define BAUD_2_4_K 0x27     //2.4 KBaud
#define BAUD_4_8_K 0x37
#define BAUD_9_6_K 0x47
#define BAUD_19_2_K 0x57
#define BAUD_38_4_K 0x57

#define BAUD_19_2_K_LOWER_TEST4 0x25

// Defition for different working mode for the cc1000
#define CC1000_RX_MODE 0x00
#define CC1000_TX_MODE 0x01
#define CC1000_PD_MODE 0x02
#define CC1000_SLEEP_MODE 0x03

#endif
