/*
 * Copyright (C) 2000-2004 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */
 
/*
 * CC1000 Frequency Hopping Library
 *
 * This library allows for fast channel switching of pre-configured channels
 *
 * Usage:
 * - cc1000_freqhop_init defines the used channels
 * - cc1000_freqhop_rxmode and cc1000_freqhop_txmode are used to switch to the specified
 *         and sets the radio mode
 *
 * Author: Matthias Ringwald <mringwal@inf.ethz.ch>
 *
 */

#include <sys/types.h>

/**
 * Init frequency table and calibrate each RX/TX pair
 *
 * @param frequency number of channels to be used
 * @param frequency of channel #0
 * @param frequency seperation between two adjunct channels
 *
 * @note  It is assumed that this function is only called once
 *        therefore it does not bother with memory freeing
 *
 * @pre   cc1000 is initialized
 * @post  calibrated frequency table is ready
 */
void cc1000_freqhop_init(u_char nr, u_long base, u_long seperation);

/**
 * Set channel for receive
 *
 * @param channel nr
 * @pre   cc1000_freqhop_init(channels, ..) was called and nr < channels
 * @pre   basic configuration was already done
 * @post  cc1000 registers are set. mode is not set
 */
void cc1000_freqhop_rxmode(u_char nr);

/**
 * Set channel for transmit
 *
 * @param channel nr
 * @pre   cc1000_freqhop_init(channels, ..) was called and nr < channels
 * @pre   basic configuration was already done
 * @post  cc1000 registers are set. mode is not set
 */
void cc1000_freqhop_txmode(u_char nr);

/** 
 * Get access to calibrated channel struct *
 *
 * @param channel nr
 * @return pointer to requested channel struct 
 */
struct cc_channel * getChannel(u_char nr);

