/*
 * Copyright (C) 2005 Mario Strasser <mast@gmx.net>,
 *                    Swiss Federal Institute of Technology (ETH) Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: ccc_packet.h,v 1.2 2007/11/05 16:37:57 freckle Exp $
 */

/**
 * @file ccc_packet.h
 * @author Mario Strasser <mast@gmx.net>
 * @author Matthias Ringwald <mringwal@inf.ethz.ch>
 *
 * @brief Packet struct for Packet-communication library for the CC1000 radio.
 */
 
#ifndef _CCOMPACKET_H_
#define _CCOMPACKET_H_
 
#include <sys/types.h>


/**
 * Structure of a CC1000 communication (ccc) packet.
 */
typedef struct ccc_packet_st {
    /** source of the packet */
    u_short src;
    /** destination of the packet */
    u_short dst; 
    /** payload length */
    u_short length;
    /** packet type */
    u_char type;
	/** flags */
	u_char flags;
	/** local time at send */
	u_long send_time;
	/** local time at receive */
	u_long receive_time;
    /** payload data */
    u_char data[0];
} __attribute__((__packed__)) ccc_packet_t;

#define CCC_FLAG_TIMESTAMP 0x01

#endif /* _CCOMPACKET_H_ */
