/*
 * Copyright (C) 2000-2006 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */

/*!
 * \file con_event_buffer.h
 * \brief buffers conection table change events comming from btstack.
 * 
 * This file provides a mechanism to buffer connection-table-change-events from
 * the btstack. This is needed since the callback call from the btstack should
 * return as soon as possible, running on the high priority btstack thread.
 * Further event handling is done in a seperate thread which calls 
 * #con_event_listen(). 
 */


#include <bt/bt_defs.h>

#define MAX_CON_CHANGE_EVENT_ENTRY 10   //what size has to be???

typedef struct _event_entry {
    u_char type;
    u_char detail;
    bt_hci_con_handle_t con_handle;
} event_entry_t;

typedef struct _event_fifo {
   event_entry_t event_list[MAX_CON_CHANGE_EVENT_ENTRY];
   u_char rd_idx;
   u_char wr_idx;
   u_char isfull;
} event_fifo_t;

typedef struct _con_event_info {
    HANDLE table_changed_event;
    event_fifo_t event_fifo;
    HCI_CON_TABLE_CB;
    void* cb_arg;
    char debug_open_con;
    struct btstack* bt_stack;
} con_event_info_t;

con_event_info_t* _con_event_info;

/*!
 * \brief initializes the connection event buffer and registers itself to the
 * btstack.
 * \param bt_stack pointer to the btstack.
 */
void con_event_buffer_init(struct btstack *bt_stack, HCI_CON_TABLE_CB_REGISTRATION);


/*!
 * Registers external function cb, wich will be called on connection table 
 * changes. Function cb will run in the context of the thread which
 * called #con_event_listen().
 */
void con_event_buffer_register_con_table_cb(struct btstack* bt_stack, HCI_CON_TABLE_CB, void* cb_arg);

/*!
 * \brief starts listening to connection table changes.
 * This function adjusts the connection properties and calls the registered callback
 * function.
 * \warning this function never returns!
 */ 
void con_event_listen(void);




