
/*
 * ram_banks.h
 *
 * 2003. 1.10 Urs Frey <ufrey@ee.ethz.ch>
 * 19.05.2004 Martin Hinz <btnode@hinz.ch>
 *
 * This file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 *
 * functions to write and read from additional RAM banks (addr>64kB)
 *
 *
 *  */

#ifndef _RAMBANKS_H_
#define _RAMBANKS_H_

// -------------------------------------------------------------------------
// Includes
// -------------------------------------------------------------------------
//#include <ctype.h>
//#include <stdlib.h>

//#include <btnode.h>
//#include <drivers.h>
#include <sys/types.h>


// -------------------------------------------------------------------------
// Globals
// -------------------------------------------------------------------------
#if defined(__linux__) || defined (__APPLE__)

u_char bank1[0x10000 - 0x1100];
u_char bank2[0x10000 - 0x1100];
u_char bank3[0x10000 - 0x1100];

#endif

// -------------------------------------------------------------------------
// Macros
// -------------------------------------------------------------------------

#define MIN(a,b) ((a) < (b) ? (a) : (b))

#define XRAM_BANK_SIZE 0xEF00UL

// -------------------------------------------------------------------------
// Functions
// -------------------------------------------------------------------------
/** @brief Write data to the external memory.
*  
* If external memory is available the data in the buffer with length len
* will be written to the external memory starting at address.
*
* \param buffer Data to be written.
* 
* \param address Starting address of external memory where the data will be written.
* 
* \param len Length of the data.
*
* \return 0 if data written, 1 if not successful (e.g. external memory not available).
*/
u_char cpy_to_xbank(void *buffer, u_long address, u_short len);

/** @brief Read data from the external memory.
*  
* If external memory is available the data of length len starting at address in 
* the external memory will be read into the buffer.
*
* \param buffer To store the read data.
* 
* \param address Starting address of the data in external memory.
* 
* \param len Length of the data.
*
* \return 0 if data read, 1 if not successful (e.g. external memory not available).
*/
u_char cpy_from_xbank(void *buffer, u_long address, u_short len);

/** \brief Length of the CRC check field (at the end of the memory). */
#define CRC_LENGTH 2

/** @brief Computes the CRC of a memory block.
*  
* This function computes the CRC of the memory block starting
* at start address and ending at end address and returns the value.
*
* \param start Starting memory address.
* 
* \param end Ending memory address.
*
* \return CRC as unsigned short.
*/
u_short get_xbank_crc(u_long start, u_long end);

/** @brief checks if the CRC field located after the end mark is correct.
*  
* This function checks if the CRC of the memory block starting
* at start address and ending at end address is correct. Remember, due to the 
* crc field the memory block used is larger by CRC_LENGTH bytes!
*
* \param start Starting memory address.
* 
* \param end Ending memory address (before the CRC field)
*
* \return 0 if CRC is correct, otherwise 1.
*/
u_char check_xbank_crc(u_long start, u_long end);

/** @brief updates the CRC field located after the end mark.
*  
* This function computes the CRC of the memory block starting
* at start address and ending at end address and stores the value into
* the CRC field located behind the end address.
* The CRC field has a length of CRC_LENGTH bytes!
*
* \param start Starting memory address.
* 
* \param end Ending memory address (before the CRC field)
*
*/
void update_xbank_crc(u_long start, u_long end);

#endif
