/*
 * Copyright (C) 2006 Florian Schuetz <fschuetz@ethz.ch>,
 *                    Swiss Federal Institute of Technology (ETH) Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * For additional information see http://www.btnode.ethz.ch/
  *
 * $Id: aodv-cmds.h,v 1.1 2007/06/08 09:51:39 bosterm Exp $
 */

/**
 * 
 * @file aodv-cmds.h
 * @author Florian Schuetz <fschuetz@ethz.ch>
 *
 * @brief Provides terminal commands for aodv control.
 *
 * Commands:
 *
 *  - aodv route discover (to discover a route)
 *  - !aodv send rreq (to send a route request)
 *  - !aodv send rrep (to send a route reply)
 *  - !aodv send rrep-ack (to send a route reply ack)
 *
 *  - aodv print rt (to print the routing table)
 *  - aodv print rt entry (to print a single entry from the routing table)
 *  - aodv print stats (to print statistics)
 *
 *  - !aodv set sequence no (sets the sequence number)
 *  - !aodv set rreq id (sets the route request id)
 *  - !aodv set addr (sets the mac address)
 *
 *  - aodv get addr (gets the configured mac address)
 *  - aodv get rreq_id (gets the current route request id)
 *  - aodv get sequence no (gets the current sequence number)
 *
 * ! = Dangerous, will most likely mess up routing. These commands are only available in debugmode.
 *
 * @see \link btn-terminal.h btn-terminal.h \endlink
 */

/**
 * Registers the commands for the terminal. 
 * 
 */
void aodv_cmds_register_cmds(void);
