#ifndef BTNTIME_H_
#define BTNTIME_H_

#include <sys/types.h>

#define BTN_TIME_SYNC_LEAD		0
#define BTN_TIME_SYNC_LAG		1

inline u_long btn_time_local(void);

inline u_long btn_time_global(void);

inline void btn_time_global_set_offset(u_long offset, u_char sign);

/**
 * macro to track rtc ticks when disabling interrupts for more than 1 ms
 * NOTE: macro ensures code is really inlined
 * NOTE: no need for OR as only '1' bit have effects in this register
 */
#define TRACKTIMER(counter) \
{                           \
    if ( TIFR & BV(OCF0) ){ \
        counter++;          \
		TIFR = BV(OCF0);    \
    }                       \
};

/**
 * Get time of RTC including Jiffies from interrupt
 *  
 * @note ticks 32768 hz ticks ~ 30.5 us 
 * @note must be called in atomic context
 */
inline u_long btn_get_rtc_from_irq(void);

/**
 * Get time of RTC including Jiffies
 *  
 * @note ticks 32768 hz ticks ~ 30.5 us 
 */
u_long btn_get_rtc(void);

#endif /*BTNTIME_H_*/
