#
# Copyright (C) 2001-2002 by egnite Software GmbH. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this
#    software must display the following acknowledgement:
#
#    This product includes software developed by egnite Software GmbH
#    and its contributors.
#
# THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
# SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
# THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# For additional information see http://www.btnode.ethz.ch/
#
# $Log: Makefile.os,v $
# Revision 1.8  2007/02/02 10:28:40  yuecelm
# minor changes
#
# Revision 1.7  2005/02/25 12:26:24  phblum
# merged in changes from branch moveup_NUT_CVS. main purpose: change location of nut in the file hierarchy
#
# Revision 1.5.2.2  2005/02/18 12:50:47  dyerm
# removed top_srcdir variable definition. Its now exported by parent makefile.
#
# Revision 1.5.2.1  2005/01/27 08:51:37  phblum
# changed top_srcdir for new nut location, include tracer.c when making libnutos
#
# Revision 1.5  2004/10/28 14:37:27  freckle
# fixed clean target to clean ALL multiplatform generated code
#
# Revision 1.4  2004/08/25 10:43:15  freckle
# Added defines for mica2 platform
#
# Revision 1.3  2004/06/10 12:37:57  freckle
# Started to use new multi-platform Makefiles
#
# Revision 1.2  2004/06/08 17:54:45  freckle
# Added Makefile.dev to build most wanted nut/os devs
# Fixed smaller bugs in the Makefiles
# Introduced TARGET folders in btnut/lib and made makefiles put libs there
#
# Revision 1.1  2004/06/07 16:39:11  freckle
# Added btnode/nut/Makefile[.os] to compile nut/os from outside, tailored
# to our multi-platform support.
#
#

# define current module
PROJ = libnutos

include $(top_srcdir)/Makedefs

# 
PROJTYPE = nutlib

# define sources
SRC1 = 	nutinit.c
SRC2 =  devreg.c timer.c msg.c event.c thread.c heap.c osdebug.c confos.c \
	version.c semaphore.c mutex.c tracer.c

# define object files and targets based sources
OBJ1.btnode2 = $(SRC1:.c=.btnode2.o)
OBJ2.btnode2 = $(SRC2:.c=.btnode2.o) 
TRG1.btnode2 = $(OBJ1.btnode2)
TRG2.btnode2 = $(PROJ).btnode2.a $(SRC1.c=.d)

OBJ1.btnode3 = $(SRC1:.c=.btnode3.o)
OBJ2.btnode3 = $(SRC2:.c=.btnode3.o) 
TRG1.btnode3 = $(OBJ1.btnode3)
TRG2.btnode3 = $(PROJ).btnode3.a $(SRC1.c=.d)

OBJ1.mica2 = $(SRC1:.c=.mica2.o)
OBJ2.mica2 = $(SRC2:.c=.mica2.o) 
TRG1.mica2 = $(OBJ1.mica2)
TRG2.mica2 = $(PROJ).mica2.a $(SRC1.c=.d)

OBJ1.unix = $(SRC1:.c=.unix.o)
OBJ2.unix = $(SRC2:.c=.unix.o) 
TRG1.unix = $(OBJ1.unix)
TRG2.unix = $(PROJ).unix.a $(SRC1.c=.d)

TRG1 = nutinit.o
TRG2 = $(PROJ).a 


	
# define make and make install targets
all:     $(PLATFORMS:=.all) 

install: $(PLATFORMS:=.install)


include $(top_srcdir)/Makerules



# define unix target
$(TRG1.unix): $(SRC1.c=.d)
$(TRG2.unix): $(OBJ2.unix) $(SRC2.c=.d)
	$(AR) $(ARFLAGS.UNIX) $(TRG2.unix) $(OBJ2.unix)

unix.all:    $(TRG1.unix) $(TRG2.unix)

unix.install: $(TRG1.unix) $(TRG2.unix)
	$(MKDIR) $(LIBDIR.UNIX)
	$(CP) $(TRG1.unix) $(LIBDIR.UNIX)/$(TRG1)
	$(CP) $(TRG2.unix) $(LIBDIR.UNIX)/$(TRG2)
	$(RANLIB) $(LIBDIR.UNIX)/$(TRG2)


#define target btnode2
$(TRG1.btnode2): $(SRC1.c=.d)
$(TRG2.btnode2): $(OBJ2.btnode2) $(SRC2.c=.d)
	$(AVR-AR) $(ARFLAGS.BTNODE2) $(TRG2.btnode2) $(OBJ2.btnode2)

btnode2.all: $(TRG1.btnode2) $(TRG2.btnode2)

btnode2.install: $(TRG1.btnode2) $(TRG2.btnode2)
	$(MKDIR) $(LIBDIR.BTNODE2)
	$(CP) $(TRG1.btnode2) $(LIBDIR.BTNODE2)/$(TRG1)
	$(CP) $(TRG2.btnode2) $(LIBDIR.BTNODE2)/$(TRG2)
	$(AVR-RANLIB) $(LIBDIR.BTNODE2)/$(TRG2)


#define target btnode3
$(TRG1.btnode3): $(SRC1.c=.d)
$(TRG2.btnode3): $(OBJ2.btnode3) $(SRC2.c=.d)
	$(AVR-AR) $(ARFLAGS.BTNODE3) $(TRG2.btnode3) $(OBJ2.btnode3)

btnode3.all: $(TRG1.btnode3) $(TRG2.btnode3)

btnode3.install: $(TRG1.btnode3) $(TRG2.btnode3)
	$(MKDIR) $(LIBDIR.BTNODE3)
	$(CP) $(TRG1.btnode3) $(LIBDIR.BTNODE3)/$(TRG1)
	$(CP) $(TRG2.btnode3) $(LIBDIR.BTNODE3)/$(TRG2)
	$(AVR-RANLIB) $(LIBDIR.BTNODE3)/$(TRG2)


#define target mica2
$(TRG1.mica2): $(SRC1.c=.d)
$(TRG2.mica2): $(OBJ2.mica2) $(SRC2.c=.d)
	$(AVR-AR) $(ARFLAGS.MICA2) $(TRG2.mica2) $(OBJ2.mica2)

mica2.all: $(TRG1.mica2) $(TRG2.mica2)

mica2.install: $(TRG1.mica2) $(TRG2.mica2)
	$(MKDIR) $(LIBDIR.MICA2)
	$(CP) $(TRG1.mica2) $(LIBDIR.MICA2)/$(TRG1)
	$(CP) $(TRG2.mica2) $(LIBDIR.MICA2)/$(TRG2)
	$(AVR-RANLIB) $(LIBDIR.MICA2)/$(TRG2)



clean:
	-rm -f $(OBJ1.unix) $(OBJ2.unix)
	-rm -f $(TRG1.unix) $(TRG2.unix)
	-rm -f $(OBJ1.btnode2) $(OBJ2.btnode2)
	-rm -f $(TRG1.btnode2) $(TRG2.btnode2)
	-rm -f $(OBJ1.btnode3) $(OBJ2.btnode3)
	-rm -f $(TRG1.btnode3) $(TRG2.btnode3)
	-rm -f $(OBJ1.mica2) $(OBJ2.mica2)
	-rm -f $(TRG1.mica2) $(TRG2.mica2)
	-rm -f $(SRC1:.c=.btnode2.lst) $(SRC2:.c=.btnode2.lst)
	-rm -f $(SRC1:.c=.btnode2.lis) $(SRC2:.c=.btnode2.lis)
	-rm -f $(SRC1:.c=.btnode3.s) $(SRC2:.c=.btnode3.s)
	-rm -f $(SRC1:.c=.btnode3.lst) $(SRC2:.c=.btnode3.lst)
	-rm -f $(SRC1:.c=.unix.lst) $(SRC2:.c=.unix.lst)
	-rm -f $(SRC1:.c=.unix.lis) $(SRC2:.c=.unix.lis)
	-rm -f $(SRC1:.c=.mica2.s) $(SRC2:.c=.mica2.s)
	-rm -f $(SRC1:.c=.mica2.lst) $(SRC2:.c=.mica2.lst)
	-rm -f $(SRC1:.c=.bak) $(SRC2:.c=.bak)
	-rm -f $(SRC1:.c=.c~) $(SRC2:.c=.c~)
