/*
 * Copyright (C) 2006 Florian Schuetz <fschuetz@ethz.ch>,
 *                    Swiss Federal Institute of Technology (ETH) Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * For additional information see http://www.btnode.ethz.ch/
 * 
 * $Id: aodv-cmds.c,v 1.3 2007/06/11 12:05:41 yuecelm Exp $
 */

#ifdef __BTNODE3__


#include <string.h>
#include <terminal/btn-terminal.h>
#include <sys/heap.h>
#include <sys/timer.h>
#include <cc/aodv.h>
#include <terminal/aodv-cmds.h>

#define NODE_ADDRESS 		0x0001
#define PACKET_TYPE     	0x01
#define MAX_PACKET_SIZE         8


///////////////////////////////////////////
// Route commands
///////////////////////////////////////////

void _adov_cmds_aodv_routing_usage(void) 
{
  printf("Usage:\n"
	 "\taodv route discover <dst>\n\n"
	 );
}

/**
 * Discovers a route to a given destination
 *
 * @param arg [in] pointer to the remaining
 *                 command argument string.
 */
void _aodv_cmds_aodv_route_discover(u_char *arg) 
{
  int dst;

  if(sscanf((char *) arg, "%u", &dst) == 1) {    
    if(aodv_route_discover(dst) == 0) {
      printf("Initiated route discovery to %u.\n",dst);
    } else {
      printf("Could not initiate route discovery to %u.\n", dst);
    }      
  } else {
    printf("Usage:\n"
	   "\taodv route discover <dst>\n\n");
  }
}


void _aodv_cmds_route_cmd(u_char *arg)
{	
  if(!strncmp((char *) arg, "discover", 8)) {
    _aodv_cmds_aodv_route_discover(arg+9);
  } else {
    _adov_cmds_aodv_routing_usage();
  }
}


///////////////////////////////////////////
// Send commands
//////////////////////////////////////////

void _adov_cmds_aodv_send_usage(void) 
{
  printf("Usage:\n"
#ifdef ROUTING_DEBUGMODE
	 "\t!aodv send rreq <flags> <hopcount> <reserved> <id>\n"
	 "\t\t<destination_address> <dsn>\n"
	 "\t\t<originator_address> <osn> <ttl>\n"
	 "\t!aodv send rrep     <flags> <org> <ttl>\n"
         "\t!aodv send rrep-ack <dst>\n"
	 "\n"
	 "! = Dangerous, will mess up routing if not applied carefully\n\n" 
#else
	 "\t*aodv send rreq <flags> <hopcount> <reserved> <id>\n"
	 "\t\t<destination_address> <dsn>\n"
	 "\t\t<originator_address> <osn> <ttl>\n"
	 "\t*aodv send rrep     <flags> <org> <ttl>\n"
         "\t*aodv send rrep-ack <dst>\n"     
	 "\n"
	 "* = Deactivated, enable Debugmode to use these commands!\n"
#endif //ROTUING_DEBUGMODE
	 );
}


#ifdef ROUTING_DEBUGMODE

void _aodv_cmds_aodv_send_rreq(u_char *arg) 
{
  int i;
  int flags, hopcount, reserved;
  int id, destination_address, dsn, originator_address, osn, ttl;

  if(sscanf(arg, "%u %u %u %u %u %u %u %u %u", &flags, &hopcount, &reserved, &id, &destination_address,
    &dsn, &originator_address, &osn, &ttl) == 9) {    

    i = aodv_debug_send_rreq_packet((u_char) flags, (u_char) hopcount,
				    (u_char) reserved, (u_short) id, (u_short) destination_address,
				    (u_short) dsn, (u_short) originator_address, (u_short) osn,
				    (u_short) ttl);
    if(i == 0) {
      printf("RREQ succesfully sent on its way.\n");
    } else {
      printf("Could not send RREQ.\n");
    }
  } else {
    printf("Usage:\n"
	   "\taodv send rreq <flags> <hopcount> <reserved> <id>\n"
	   "\t\t<destination_address> <dsn>\n"
	   "\t\t<originator_address> <osn> <ttl>\n");
  }
}

void _aodv_cmds_aodv_send_rrep(u_char *arg) 
{
  printf("TODO\n");
}

void _aodv_cmds_aodv_send_rrep_ack(u_char *arg) 
{
  printf("TODO\n");
}

#endif //ROUTING_DEBUGMODE


void _aodv_cmds_send_cmd(u_char *arg)
{	
  if(!strncmp((char *) arg, "rreq", 4)) {
#ifdef ROUTING_DEBUGMODE
    _aodv_cmds_aodv_send_rreq(arg+5);
#else
    printf("Deactivated, enable Debugmode to use this command!\n");
#endif //ROUTING_DEBUGMODE
  } else if(!strncmp((char *) arg, "rrep", 4)) {
#ifdef ROUTING_DEBUGMODE
    _aodv_cmds_aodv_send_rrep(arg+5);
#else
    printf("Deactivated, enable Debugmode to use this command!\n");
#endif //ROUTING_DEBUGMODE
  } else if(!strncmp((char *) arg, "rrep-ack",8)) {
#ifdef ROUTING_DEBUGMODE
    _aodv_cmds_aodv_send_rrep_ack(arg+9);
#else
    printf("Deactivated, enable Debugmode to use this command!\n");
#endif //ROUTING_DEBUGMODE
  } else {
    _adov_cmds_aodv_send_usage();
  }
}


///////////////////////////////////////////////
// Print commands
//////////////////////////////////////////////

void _adov_cmds_aodv_print_usage(void) 
{
  printf("Usage:\n"
	 "aodv print rt\n"
	 "aodv print rt entry <no>\n"
	 "aodv print stats");
}

void _aodv_cmds_aodv_print_stats(void)
{ 
  printf("           Recv    Send  RecvFail SendFail Ignored\n");
  printf("RREQ      %5u   %5u     %5u    %5u   %5u\n",
	 aodv_rreq_recv_count, aodv_rreq_send_count, aodv_rreq_recv_fail_count,
	 aodv_rreq_send_fail_count, aodv_rreq_ignore_count);
  printf("RREP      %5u   %5u     %5u    %5u       -\n",
	 aodv_rrep_recv_count, aodv_rrep_send_count, aodv_rrep_recv_fail_count,
	 aodv_rrep_send_fail_count);
  printf("RREP-ACK  %5u   %5u         -    %5u       -\n",
	 aodv_rrep_ack_recv_count, aodv_rrep_ack_send_count, aodv_rrep_ack_send_fail_count);
  printf("RERR      %5u   %5u         -    %5u       -\n",
	 aodv_rerr_recv_count, aodv_rerr_send_count, aodv_rerr_send_fail_count);
}

void _aodv_cmds_aodv_print_rt_entry(char *arg)
{ 
  int val;
  if(sscanf(arg, "%u", &val) == 1) {
    aodv_print_rt_entry(val);
  } else {
    printf("Usage: aodv print rt entry <num>\n");
  }
}

void _aodv_cmds_aodv_print_cmd(char *arg)
{
  if(!strncmp(arg, "rt entry", 8)) {
    _aodv_cmds_aodv_print_rt_entry(arg+9);
  } else if(!strncmp(arg, "rt", 2)) {
    aodv_print_rt();
  } else if(!strncmp(arg, "stats", 5)) {
    _aodv_cmds_aodv_print_stats();
  } else {
    _adov_cmds_aodv_print_usage();
  }
}


//////////////////////////////////////
// Setters
//////////////////////////////////////

void _aodv_cmds_set_print_usage(void)
{
  printf("'Usage:\n"
#ifdef ROUTING_DEBUGMODE
	 "\t!aodv set sequence no <seqno>\n"
	 "\t!aodv set rreq id <rreq id>\n"
         "\t!aodv set addr <addr>\n"
	 "\n"
	 "! = Dangerous, will mess up routing if not applied carefully\n\n" 
#else
	 "\t*aodv set sequence no <seqno>\n"
	 "\t*aodv set rreq id <rreq id>\n"
         "\t*aodv set addr <addr>\n"
	 "\n"
	 "* = Deactivated, enable Debugmode to use these commands!\n"
#endif //ROTUING_DEBUGMODE
	 );
}

#ifdef ROUTING_DEBUGMODE
void _aodv_cmds_set_seqno(char *arg)
{
  int val;
  if(sscanf(arg, "%u", &val) == 1) {
    aodv_set_sequence_no(val);
    printf("Node Sequence Number set to %u\n\n", aodv_get_sequence_no());
  } else {
    printf("Usage: aodv set sequence no <seqno>\n");
  }
}

void _aodv_cmds_set_rreq_id(char *arg)
{
  int val;
  if(sscanf(arg, "%u", &val) == 1) {
    aodv_set_rreq_id(val);
    printf("Node RREQ Id set to %u\n\n", aodv_get_rreq_id());
  } else {
    printf("Usage: aodv set rreq id <rreq id>\n");
  }
}
#endif //ROUTING_DEBUGMODE

void _aodv_cmds_set_addr(char *arg)
{ 
  int val;
  if(sscanf(arg, "%u", &val) == 1) {
    printf("Setting new node address to %u.\n\n",val);
    aodv_set_address(val);
  } else {
    printf("Usage: aodv set addr <addr>\n");
  }
}

void _aodv_cmds_aodv_set_cmd(char* arg) {
  if (!strncmp(arg, "addr", 4)) {
#ifdef ROUTING_DEBUGMODE
    _aodv_cmds_set_addr(arg+5);
#else
    printf("Deactivated, enable Debugmode to use this command!\n");
#endif //ROUTING_DEBUGMODE
  } else if (!strncmp(arg, "rreq id", 7)) {
#ifdef ROUTING_DEBUGMODE
    _aodv_cmds_set_rreq_id(arg + 8);
#else
    printf("Deactivated, enable Debugmode to use this command!\n");
#endif //ROUTING_DEBUGMODE
  } else if (!strncmp(arg, "sequence no", 11)) {
#ifdef ROUTING_DEBUGMODE
    _aodv_cmds_set_seqno(arg + 12);
#else
    printf("Deactivated, enable Debugmode to use this command!\n");
#endif //ROUTING_DEBUGMODE
  } else {
    _aodv_cmds_set_print_usage();
  }
}

///////////////////////////////////////
// Getters
///////////////////////////////////////
void _aodv_cmds_get_print_usage(void)
{
  printf("Usage:\n"
	 "\taodv get addr\n"
	 "\taodv get rreq id\n"
	 "aodv get sequence no\n\n");
}

void _aodv_cmds_get_seqno(void)
{
  printf("Node Sequence Number: %u\n\n", aodv_get_sequence_no());
}

void _aodv_cmds_get_rreq_id(void)
{
  printf("Node RREQ Id: %u\n\n", aodv_get_rreq_id());
}


void _aodv_cmds_get_addr(void)
{
  printf("Node Address: %u\n\n", aodv_get_address());
}

void _aodv_cmds_aodv_get_cmd(char* arg) {
  if (!strncmp(arg, "addr", 4)) {
    _aodv_cmds_get_addr();
  } else if (!strncmp(arg, "rreq id", 7)) {
    _aodv_cmds_get_rreq_id();
  } else if (!strncmp(arg, "sequence no", 11)) {
    _aodv_cmds_get_seqno();
  } else {
    _aodv_cmds_get_print_usage();
  }
}


///////////////////////////////////////////
// AODV Command main
//////////////////////////////////////////

void _aodv_cmds_print_usage(void)
{
  printf("Usage:\n"
	 "\taodv send <type> <...>\n"
	 "\taodv set <param> <...>\n"
	 "\taodv get <param> <...>\n"
	 "\taodv print <param> [<...>]\n"
	 "\taodv route discover <dst>\n\n");
}

void _aodv_cmds_aodv_cmd(char* arg) {
  if (!strncmp(arg, "route", 5)) {
    _aodv_cmds_route_cmd((u_char *) arg+6);
  } else if (!strncmp(arg, "send", 4)) {
    _aodv_cmds_send_cmd((u_char *) arg+5);
  } else if (!strncmp(arg, "set", 3)) {
    _aodv_cmds_aodv_set_cmd(arg+4);
  } else if (!strncmp(arg, "get", 3)) {
    _aodv_cmds_aodv_get_cmd(arg+4);
  } else if (!strncmp(arg, "print", 5)) {
    _aodv_cmds_aodv_print_cmd(arg+6);
  } else {
    _aodv_cmds_print_usage();
  }
}

void aodv_cmds_register_cmds(void)
{
    btn_terminal_register_cmd("aodv", _aodv_cmds_aodv_cmd);
}

#endif
