/*
 * Copyright (C) 2006 Florian Schuetz <fschuetz@ethz.ch>,
 *                    Swiss Federal Institute of Technology (ETH) Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * For additional information see http://www.btnode.ethz.ch/
 * 
 * $Id: rfc-cmds.c,v 1.3 2007/06/11 12:05:41 yuecelm Exp $
 */

#ifdef __BTNODE3__

#include <string.h>
#include <terminal/btn-terminal.h>
#include <sys/event.h>
#include <sys/heap.h>
#include <sys/timer.h>
#include <sys/types.h>

#include <cc/rfc.h>
#include <terminal/aodv-cmds.h>

#define NODE_ADDRESS 		0x0001
#define PORT    	        0x01
#define MAX_PACKET_SIZE         8

rfc_packet_t *pkt;
int run, thread_sleep;
u_short thread_dst;
HANDLE event_mping;

/**
 * Sends a ping packet after sleeping thread_sleep milliseconds
 */
THREAD(rfc_cmd_mping_thread, arg)
{
  rfc_packet_t *packet = new_rfc_packet(sizeof(int));
  int count = 1;  
  packet->port = 1;
  packet->length = sizeof(int);

  for(;;) {
    if(run == 0) {
      count = 0;
      NutEventWait(&event_mping, NUT_WAIT_INFINITE);
    } else {    
      printf("Sending Packet number %u to %u. (Interval: %u)\n",count,thread_dst, thread_sleep);
      *((int*) packet->data) = htons((int) count);

      if(thread_dst != 0 && rfc_send(thread_dst, PORT, packet) == -1) {
	//Sending failed
	printf("\nE: Could not send %u to %u\n", count, thread_dst);
      }
      count++;
      NutEventWait(&event_mping, thread_sleep);
    }
  }
}

/**
 * Sends multiple packets to a given desitnation
 */
void _rfc_cmds_mping_cmd(char *arg)
{
  int sleep = 0;
  u_short dst;

  if(!strncmp(arg, "start", 5) &&
     (sscanf(arg+6, "%u%hd", &sleep,&dst) == 2)) {
    printf("Starting thread.\n");
    thread_sleep = sleep;
    thread_dst = dst;
    run = 1;
    NutEventPost(&event_mping);
  } else if(!strncmp(arg, "stop", 4)) {
    printf("Stopping thread.\n");
    run = 0;
  } else {
    printf("rfc mping start <sleeptime> <dest>\n"
	   "rfc mping stop\n");
  }
}

/**
 * Sends a number to a given destination
 */
void _rfc_cmds_send(u_char *arg)
{	
  int val, node_address;

  // Replace by new_rfc_packet
  pkt = (rfc_packet_t *)new_rfc_packet(sizeof(int));
  pkt->port = 1;
  pkt->length = sizeof(int);

  if(sscanf((char *) arg, "%u %u", &node_address, &val) == 2) {
    *((int*) pkt->data) = htons((int) val);

    if(rfc_send(node_address, PORT, pkt) == -1) { 
      /* send failed */ 
      printf("\nE: Could not send %u to %u\n", val, node_address);
    }
  } else {
    printf("Usage: aodv send <destination> <value>.\n");
  }
					       
  free(pkt);
}


/**
 * Obsolete command to set the mac address. Use cc1000 command instead.
 */
void _rfc_cmds_set_addr(char *arg)
{ 
  int val;
  if(sscanf(arg, "%u", &val) == 1) {
    printf("Obsolete. Use cc1000 command for this and reboot!\n");
  } else {
    printf("Usage: rfc set addr <addr>\n");
  }
}

void _rfc_cmds_get_print_usage(void)
{
  printf("rfc get: error usage:\n"
	  " rfc get <addr>\n");
}

void _rfc_cmds_rfc_get_cmd(char* arg) {
  if (!strncmp(arg, "addr", 4)) {
    printf("Obsolete. Use cc1000 command for this.\n");
  } else {
    _rfc_cmds_get_print_usage();
  }
}

void _rfc_cmds_set_print_usage(void)
{
  printf("rfc set: error usage:\n"
	  " rfc set <addr>\n");
}

void _rfc_cmds_rfc_set_cmd(char* arg) {
  if (!strncmp(arg, "addr", 4)) {
    _rfc_cmds_set_addr(arg+5);
  } else {
    _rfc_cmds_set_print_usage();
  }
}

void _rfc_cmds_print_usage(void)
{
  printf("rfc: error usage:\n"
	 " rfc send <addr> <number>\n"
	 " rfc mping start <sleeptime> <dest>\n"
	 " rfc mping stop\n"
	 " rfc set <param> <...>\n"
	 " rfc get <param> <...>\n");
}


void _rfc_cmds_rfc_cmd(char* arg) {
  if (!strncmp(arg, "send", 4)) {
    _rfc_cmds_send((u_char *) arg+5);
  } else if (!strncmp(arg, "mping", 5)) {
    _rfc_cmds_mping_cmd(arg+6);
  } else if (!strncmp(arg, "set", 3)) {
    _rfc_cmds_rfc_set_cmd(arg+4);
  } else if (!strncmp(arg, "get", 3)) {
    _rfc_cmds_rfc_get_cmd(arg+4);
  } else {
    _rfc_cmds_print_usage();
  }
}

/** 
 * Error handler that registers to RFC.
 */
void rfc_cmds_error_handler(u_short error, u_short seqno)
{
  printf("Error %u for packet %u occured!\n\n",error, seqno);
}

/**
 * Packet handler that listens for incomming packets.
 */
void rfc_cmds_packet_handler(rfc_packet_t *pkt)
{
  int val;

  val = ntohs(*((int*)pkt->data));
  printf("\n### Received packet from 0x%04X: %u\n", pkt->src, val);
}

/**
 * Prepares and registers the commands to the terminal.
 */
void rfc_cmds_register_cmds(void)
{
  rfc_packet_handler_t handler;

  //Prepare commands
  handler.error_handler = rfc_cmds_error_handler;
  handler.packet_handler = rfc_cmds_packet_handler;
  rfc_register_packet_handler(1, &handler);

  //Prepare thread
  run = 0;
  if( 0 == NutThreadCreate("rfc_cmdp", rfc_cmd_mping_thread, 0 , 512)) { //IMPROVE - Decide minimal stack size
    //Creating Thread failed, it will not operate
    ROUTING_DEBUG("\nE: (rfc_cmds_register_cmds) Create mping thread failed!\n");
  }

  //Register Terminal commands
  btn_terminal_register_cmd("rfc", _rfc_cmds_rfc_cmd);
}

#endif
