#include <time/btn-time.h>
#include <sys/timer.h>
#include <sys/atom.h>

/* NutOS system ticks */
extern u_long nut_ticks;

long sync_offset = 0;
u_char sync_state;

inline u_long btn_time_local(void)
{
	return NutGetMillis();
}

inline u_long btn_time_global(void)
{
	if (sync_state == BTN_TIME_SYNC_LAG)
	{
		return NutGetMillis() + sync_offset;
	}
	else
	{
		return NutGetMillis() - sync_offset;
	}
}

inline void btn_time_global_set_offset(u_long offset, u_char sign)
{
	sync_offset = offset;
	sync_state = sign;
}


#if defined(__BTN_UNIX__)
/** crude unix implemenation */
u_long btn_get_rtc(){
	return NutGetMillis() * 32;
}
u_long btn_get_rtc_from_irq(void){
	return NutGetMillis() * 32;
}
#else

/**
 * Get time of RTC including Jiffies
 *  
 * @note ticks 32768 hz ticks ~ 30.5 us 
 * @note must be called in atomic context
 */
inline u_long btn_get_rtc_from_irq(void)
{
    u_long t_high = nut_ticks;
    u_char t_low = TCNT0;
    
    /*
     * Adjust time if we did not handle the overflow interrupt yet
     * AND the low time we read is already shows this.
     *
     * Without the second test:
     * nut_ticks = 0, t = 31, IRQ = 0. read TCNT0 = 32
     * nut_ticks = 0, t = 0,  IRQ = 1. read IRQ = 1
     * => jiffies = 63 instead of 31
     */
    if( bit_is_set(TIFR, OCF0) && (t_low <= 0x10))
        t_high++;
    
    return (t_high << 5) | t_low;
}

/**
 * Get time of RTC including Jiffies
 *  
 * @note ticks 32768 hz ticks ~ 30.5 us 
 */
u_long btn_get_rtc(void){
    u_long result;
    NutEnterCritical();
    result = btn_get_rtc_from_irq();
    NutExitCritical();
    return result;
}
#endif
